import writeXlsxFile from "write-excel-file";
import { Options } from "./ExcelInterface";

const columnNames = getColumnNames();

function getColumnNames() {
	let wordList = Array.from(new Array(26), (ele, index) => {
		return String.fromCharCode(65 + index);
	});
	let columnNames = [];
	for (let i = 0; i < 26; i++) {
		columnNames.push(wordList[i]);
	}
	for (let i = 0; i < 26; i++) {
		for (let j = 0; j < 26; j++) {
			columnNames.push(wordList[i] + wordList[j]);
		}
	}
	return columnNames;
}
function convertType(value: any, type: any) {
	switch (type) {
		case String:
			return String(value);
		case Number:
			let output = Number(value);
			if (isNaN(output)) {
				output = 0;
			}
			return output;
		case Boolean:
			return Boolean(value);
		case Array:
			return Array.isArray(value) ? value : [value];
		case Object:
			return Object(value);
		case null:
			return null;
		case undefined:
			return undefined;
		default:
			return String(value);
	}
}
const ExcelWriter = {
	getColumnNames() {
		return columnNames;
	},
	async outputExcel(headerConfig: Array<any>, data: Array<any>, options: Options) {
		let excelData: Array<any> = [];

		data.forEach((row, rowIndex) => {
			let rowObject = [] as any;
			headerConfig.forEach((headerItem, headerIndex) => {
				let column = this._dealCellData(row[headerItem.key], headerItem, options);

				rowObject.push(column);
			});
			excelData.push(rowObject);
		});

		if (!options.hasOwnProperty("headerRow")) {
			// 对象 options 上没有名为 "headerRow" 的属性
			options.headerRow = 1;
		}
		if (options.headerRow > 0) {
			let excelDataHeader = [] as any;
			headerConfig.forEach((headerItem, headerIndex) => {
				let column = this._dealCellData(headerItem.title, headerItem, options);
				column.type = String;
				column.value = headerItem.title;
				excelDataHeader.push(column);
			});
			excelData.splice(options.headerRow - 1, 0, excelDataHeader);
			//excelDataBody.unshift(excelDataHeader);
		}
		//excelData = [excelDataHeader, ...excelDataBody];
		await this._output(excelData, options);
	},

	async outputExcelByData(data: Array<any>, options: Options) {
		let columnArray = this.getColumnNames(); //await ExcelWriter.getColumnNames();
		console.log("#debug#🚀 ~ file: test.vue:36 ~ testOne2Two ~ columnArray:", columnArray);
		//rowSpan span
		let arrayOrg = data;
		console.log("#debug#🚀 ~ file: test.vue:42 ~ testOne2Two ~ arrayOrg:", JSON.stringify(arrayOrg));
		let maxRow: number = 0;
		let maxColumn: number = 0;
		arrayOrg.forEach((item) => {
			let letters = item.poi.replace(/[^a-z]/gi, "").toUpperCase();
			let column: number = parseInt(item.poi.replace(/\D/g, ""));
			let row: number = columnArray.indexOf(letters);
			if (row > maxRow) {
				maxRow = row;
			}

			if (column > maxColumn) {
				maxColumn = column;
			}
			console.log("#debug#🚀 ~ file: test.vue:47 ~ arrayOrg.forEach ~ num:", column, maxColumn);
			console.log("#debug#🚀 ~ file: test.vue:46 ~ testOne2Two ~ letters:", letters, columnArray.indexOf(letters));
		});
		let excelData = [] as any;
		console.log("#debug#🚀 ~ file: test.vue:67 ~ testOne2Two ~ maxRow:", maxRow);
		console.log("#debug#🚀 ~ file: test.vue:70 ~ testOne2Two ~ maxColumn:", maxColumn);
		for (let i = 0; i <= maxColumn; i++) {
			excelData[i] = [];
			for (let j = 0; j <= maxRow; j++) {
				excelData[i][j] = null;
			}
		}
		console.log("#debug#🚀 ~ file: test.vue:75 ~ arrayOrg.forEach ~ excelData:", excelData);
		arrayOrg.forEach((item) => {
			let letters = item.poi.replace(/[^a-z]/gi, "").toUpperCase();
			let num = item.poi.replace(/\D/g, "");
			let row = columnArray.indexOf(letters);
			excelData[num - 1][row] = item;
		});
		console.log("#debug#🚀 ~ file: test.vue:75 ~ arrayOrg.forEach ~ excelData:", excelData);

		console.log("#debug#🚀 ~ file: ExcelWriter.ts:82 ~ outputExcelByData ~ options:", options);
		if (options?.border) {
			//如果有边框
			excelData.forEach((row: any, rowIndex: number) => {
				console.log("#debug#🚀 ~ file: ExcelWriter.ts:87 ~ data.forEach ~ row:", row);
				row.forEach((cell: any, cellIndex: number) => {
					console.log("#debug#🚀 ~ file: ExcelWriter.ts:87 ~ data.forEach ~ cell:", cell);
					let borderColor = "#000000";
					if (cell) {
						if (options?.borderColor) {
							borderColor = options?.borderColor;
						}
						if (cell?.borderColor) {
							borderColor = cell?.borderColor;
						}
						cell.borderColor = borderColor;
						// cell.borderColor = options?.borderColor ?? "#000000";
						console.log("#debug#🚀 ~ file: ExcelWriter.ts:87 ~ data.forEach ~ cell.borderColor:", cell.borderColor);
					}
				});
			});
			console.log("#debug#🚀 ~ file: ExcelWriter.ts:90 ~ data.forEach ~ data:", data);
		}
		if (!options.hasOwnProperty("fileName")) {
			// 对象 options 上没有名为 "fileName" 的属性
			options.fileName = "导出.xlsx";
		}

		await writeXlsxFile(excelData, options);
	},
	//==================私有方法============================

	/**
	 * 生成单元格数据
	 * @param {*} columnCellValue
	 * @param {*} header
	 * @param {*} options
	 * @returns
	 */
	_dealCellData(columnCellValue: any, header: any, options: Options) {
		let cell = {
			value: convertType(columnCellValue, header?.type ? header?.type : String) // columnCellValue.toString().trim()
		} as any;
		if (header?.width) {
			cell.width = header.width;
		}
		if (header?.type) {
			cell.type = header.type;
		}
		if (options?.border) {
			//如果有边框
			cell.borderColor = options?.borderColor ?? "#000000";
		}
		return cell;
	},
	/**
	 * 导出excel表格
	 * @param {*} data
	 * @param {*} options
	 */
	async _output(data: Array<any>, options: Options) {
		let excelRows = [];
		data.forEach((row: Array<any>, rowIndex) => {
			row.forEach((cell, cellIndex) => {
				let column = {
					value: cell
				};
				excelRows.push(column);
			});
		});
		let fileName = options?.fileName ? options.fileName : "导出.xlsx";
		let columns = options?.columns ? options.columns : [];
		let optionsOutput = {
			columns, // (optional) column widths, etc.
			fileName
		};
		await writeXlsxFile(data, optionsOutput);
	},

	/**
	 * 导出示例
	 */
	async outputTest() {
		const HEADER_ROW = [
			{
				value: "Name",
				fontWeight: "bold"
			},
			{
				value: "Date of Birth",
				fontWeight: "bold"
			},
			{
				value: "Cost",
				fontWeight: "bold"
			},
			{
				value: "Paid",
				fontWeight: "bold"
			}
		];

		const DATA_ROW_1 = [
			[
				// "Name"
				{
					type: String,
					value: "John Smith",
					borderColor: "#000000"
				},

				// "Date of Birth"
				{
					type: Date,
					value: new Date(),
					format: "mm/dd/yyyy"
				},

				// "Cost"
				{
					type: Number,
					value: 1800
				},

				// "Paid"
				{
					type: Boolean,
					value: true
				}
			],
			[
				// "Name"
				{
					type: String,
					value: "第2行",
					borderColor: "#000000"
				},

				// "Date of Birth"
				{
					type: Date,
					value: new Date(),
					format: "yyyy-mm-dd"
				},

				// "Cost"
				{
					type: Number,
					value: 1800,
					span: 3,
					borderColor: "#ff0000"
				},

				// "Paid"
				null,
				null,
				{
					type: String,
					value: "第2行00",
					borderColor: "#000000",
					fontSize: 18
				}
			]
		];

		const data = [HEADER_ROW, ...DATA_ROW_1] as any;
		let options = {
			// columns, // (optional) column widths, etc.
			fileName: "file.xlsx"
		};
		await writeXlsxFile(data, options);
	}
};

export default ExcelWriter;
